@@ -53,8 +53,8 @@ namespace {
 inline bool
 IsSpecSrgb(const ImageSpec& spec)
 {
-    return Strutil::iequals(spec.get_string_attribute("oiio:ColorSpace"),
-                            "sRGB");
+    return equivalent_colorspace(spec.get_string_attribute("oiio:ColorSpace"),
+                                 "srgb_rec709_scene");
 }
 
 }  // namespace
@@ -497,6 +497,56 @@ ImageViewer::createActions()
     toggleAreaSampleAct->setShortcut(tr("Ctrl+A"));
     connect(toggleAreaSampleAct, SIGNAL(triggered()), this,
             SLOT(toggleAreaSample()));
+
+    closeupPixelsLabel = new QLabel(tr("# closeup pixels:"));
+    closeupPixelsBox   = new QSpinBox();
+    closeupPixelsBox->setRange(9, 25);
+    closeupPixelsBox->setValue(13);
+    closeupPixelsBox->setSingleStep(2);
+
+    QString closeupPixelsTooltip = tr(
+        "Number of pixels to show in the closeup view.");
+    closeupPixelsBox->setToolTip(closeupPixelsTooltip);
+    closeupPixelsLabel->setToolTip(closeupPixelsTooltip);
+
+    closeupAvgPixelsLabel = new QLabel(tr("# closeup avg pixels:"));
+    closeupAvgPixelsBox   = new QSpinBox();
+    closeupAvgPixelsBox->setRange(3, 25);
+    closeupAvgPixelsBox->setValue(11);
+    closeupAvgPixelsBox->setSingleStep(2);
+
+    QString closeupAvgPixelsTooltip = tr(
+        "Number of pixels to use for averaging in the closeup view.");
+    closeupAvgPixelsBox->setToolTip(closeupAvgPixelsTooltip);
+    closeupAvgPixelsLabel->setToolTip(closeupAvgPixelsTooltip);
+
+
+    rotateLeftAct = new QAction(tr("&Rotate Left"), this);
+    rotateLeftAct->setShortcut(tr("Ctrl+Shift+L"));
+    connect(rotateLeftAct, SIGNAL(triggered()), this, SLOT(rotateLeft()));
+
+    rotateRightAct = new QAction(tr("&Rotate Right"), this);
+    rotateRightAct->setShortcut(tr("Ctrl+Shift+R"));
+    connect(rotateRightAct, SIGNAL(triggered()), this, SLOT(rotateRight()));
+
+    flipHorizontalAct = new QAction(tr("&Flip Horizontal"), this);
+    connect(flipHorizontalAct, SIGNAL(triggered()), this,
+            SLOT(flipHorizontal()));
+
+    flipVerticalAct = new QAction(tr("&Flip Vertical"), this);
+    connect(flipVerticalAct, SIGNAL(triggered()), this, SLOT(flipVertical()));
+
+    // Connect signals to ensure closeupAvgPixelsBox value is always <= closeupPixelsBox value
+    connect(closeupPixelsBox, QOverload<int>::of(&QSpinBox::valueChanged),
+            [this](int value) {
+                if (closeupAvgPixelsBox->value() > value)
+                    closeupAvgPixelsBox->setValue(value);
+            });
+    connect(closeupAvgPixelsBox, QOverload<int>::of(&QSpinBox::valueChanged),
+            [this](int value) {
+                if (value > closeupPixelsBox->value())
+                    closeupAvgPixelsBox->setValue(closeupPixelsBox->value());
+            });
 }
 
 
@@ -746,6 +796,11 @@ ImageViewer::createMenus()
     toolsMenu->addAction(toggleAreaSampleAct);
     toolsMenu->addMenu(slideMenu);
     toolsMenu->addMenu(sortMenu);
+    toolsMenu->addSeparator();
+    toolsMenu->addAction(rotateLeftAct);
+    toolsMenu->addAction(rotateRightAct);
+    toolsMenu->addAction(flipHorizontalAct);
+    toolsMenu->addAction(flipVerticalAct);
 
     // Menus, toolbars, & status
     // Annotate
@@ -833,6 +888,7 @@ ImageViewer::readSettings(bool ui_is_set_up)
         maxMemoryIC->setValue(settings.value("maxMemoryIC", 2048).toInt());
     slideShowDuration->setValue(
         settings.value("slideShowDuration", 10).toInt());
+    closeupPixelsBox->setValue(settings.value("closeupPixels", 13).toInt());
 
     OIIO::attribute("imagebuf:use_imagecache", 1);
 
@@ -855,6 +911,7 @@ ImageViewer::writeSettings()
     settings.setValue("autoMipmap", autoMipmap->isChecked());
     settings.setValue("maxMemoryIC", maxMemoryIC->value());
     settings.setValue("slideShowDuration", slideShowDuration->value());
+    settings.setValue("closeupPixels", closeupPixelsBox->value());
     QStringList recent;
     for (auto&& s : m_recent_files)
         recent.push_front(QString(s.c_str()));
@@ -2419,3 +2476,82 @@ ImageViewer::areaSampleMode() const
 {
     return m_areaSampleMode;
 }
+
+
+void
+ImageViewer::rotateLeft()
+{
+    IvImage* img = cur();
+    if (!img)
+        return;
+
+    ImageSpec* spec = curspecmod();
+
+    int curr_orientation = spec->get_int_attribute("Orientation", 1);
+
+    if (curr_orientation >= 1 && curr_orientation <= 8) {
+        static int next_orientation[] = { 0, 8, 5, 6, 7, 4, 1, 2, 3 };
+        curr_orientation              = next_orientation[curr_orientation];
+        spec->attribute("Orientation", curr_orientation);
+    }
+    displayCurrentImage();
+}
+
+
+void
+ImageViewer::rotateRight()
+{
+    IvImage* img = cur();
+    if (!img)
+        return;
+
+    ImageSpec* spec      = curspecmod();
+    int curr_orientation = spec->get_int_attribute("Orientation", 1);
+
+    if (curr_orientation >= 1 && curr_orientation <= 8) {
+        static int next_orientation[] = { 0, 6, 7, 8, 5, 2, 3, 4, 1 };
+        curr_orientation              = next_orientation[curr_orientation];
+        spec->attribute("Orientation", curr_orientation);
+    }
+    displayCurrentImage();
+}
+
+
+void
+ImageViewer::flipHorizontal()
+{
+    IvImage* img = cur();
+    if (!img)
+        return;
+
+    ImageSpec* spec = curspecmod();
+
+    int curr_orientation = spec->get_int_attribute("Orientation", 1);
+
+    if (curr_orientation >= 1 && curr_orientation <= 8) {
+        static int next_orientation[] = { 0, 2, 1, 4, 3, 6, 5, 8, 7 };
+        curr_orientation              = next_orientation[curr_orientation];
+        spec->attribute("Orientation", curr_orientation);
+    }
+    displayCurrentImage();
+}
+
+
+void
+ImageViewer::flipVertical()
+{
+    IvImage* img = cur();
+    if (!img)
+        return;
+
+    ImageSpec* spec = curspecmod();
+
+    int curr_orientation = spec->get_int_attribute("Orientation", 1);
+
+    if (curr_orientation >= 1 && curr_orientation <= 8) {
+        static int next_orientation[] = { 0, 4, 3, 2, 1, 8, 7, 6, 5 };
+        curr_orientation              = next_orientation[curr_orientation];
+        spec->attribute("Orientation", curr_orientation);
+    }
+    displayCurrentImage();
+}
\ No newline at end of file
