@@ -257,20 +257,24 @@ oiio_build_platform()
     platform = "UnknownOS";
 #endif
     platform += "/";
-#if defined(__x86_64__)
+#if defined(__x86_64__) || defined(_M_AMD64)
     platform += "x86_64";
-#elif defined(__i386__)
+#elif defined(__i386__) || defined(_M_IX86)
     platform += "i386";
-#elif defined(_M_ARM64) || defined(__aarch64__) || defined(__aarch64)
+#elif defined(_M_ARM64) || defined(__aarch64__) || defined(__aarch64) \
+    || defined(__ARM_ARCH)
     platform += "ARM";
 #else
-    platform = "unknown arch?";
+    platform += "unknown arch?";
 #endif
     return platform;
 }
 
+OIIO_NAMESPACE_END
 
 
+OIIO_NAMESPACE_3_1_BEGIN
+
 void
 shutdown()
 {
@@ -326,6 +330,21 @@ log_time(string_view key, const Timer& timer, int count)
 
 
 
+bool
+attribute(string_view name, TypeDesc type, cspan<std::byte> value)
+{
+    if (value.size_bytes() != type.size()) {
+        OIIO::errorfmt(
+            "OIIO::attribute given a {}-byte span as data for a {}-byte attribute {} {}",
+            value.size(), type.size(), type, name);
+        OIIO_DASSERT(value.size_bytes() == type.size());
+        return false;
+    }
+    return attribute(name, type, value.data());
+}
+
+
+
 bool
 attribute(string_view name, TypeDesc type, const void* val)
 {
@@ -343,7 +362,7 @@ attribute(string_view name, TypeDesc type, const void* val)
     }
     if (Strutil::starts_with(name, "gpu:")
         || Strutil::starts_with(name, "cuda:")) {
-        return pvt::gpu_attribute(name, type, val);
+        return OIIO::pvt::gpu_attribute(name, type, val);
     }
 
     // Things below here need to buarded by the attrib_mutex
@@ -446,6 +465,21 @@ attribute(string_view name, TypeDesc type, const void* val)
 
 
 
+bool
+getattribute(string_view name, TypeDesc type, span<std::byte> value)
+{
+    if (value.size_bytes() != type.size()) {
+        OIIO::errorfmt(
+            "OIIO::getattribute given a {}-byte span as data for a {}-byte attribute {} {}",
+            value.size(), type.size(), type, name);
+        OIIO_DASSERT(value.size_bytes() == type.size());
+        return false;
+    }
+    return getattribute(name, type, value.data());
+}
+
+
+
 bool
 getattribute(string_view name, TypeDesc type, void* val)
 {
@@ -460,7 +494,7 @@ getattribute(string_view name, TypeDesc type, void* val)
     }
     if (Strutil::starts_with(name, "gpu:")
         || Strutil::starts_with(name, "cuda:")) {
-        return pvt::gpu_getattribute(name, type, val);
+        return OIIO::pvt::gpu_getattribute(name, type, val);
     }
 
     // Things below here need to buarded by the attrib_mutex
@@ -479,31 +513,31 @@ getattribute(string_view name, TypeDesc type, void* val)
     }
     if (name == "format_list" && type == TypeString) {
         if (format_list.empty())
-            pvt::catalog_all_plugins(plugin_searchpath.string());
+            OIIO::pvt::catalog_all_plugins(plugin_searchpath.string());
         *(ustring*)val = ustring(format_list);
         return true;
     }
     if (name == "input_format_list" && type == TypeString) {
         if (input_format_list.empty())
-            pvt::catalog_all_plugins(plugin_searchpath.string());
+            OIIO::pvt::catalog_all_plugins(plugin_searchpath.string());
         *(ustring*)val = ustring(input_format_list);
         return true;
     }
     if (name == "output_format_list" && type == TypeString) {
         if (output_format_list.empty())
-            pvt::catalog_all_plugins(plugin_searchpath.string());
+            OIIO::pvt::catalog_all_plugins(plugin_searchpath.string());
         *(ustring*)val = ustring(output_format_list);
         return true;
     }
     if (name == "extension_list" && type == TypeString) {
         if (extension_list.empty())
-            pvt::catalog_all_plugins(plugin_searchpath.string());
+            OIIO::pvt::catalog_all_plugins(plugin_searchpath.string());
         *(ustring*)val = ustring(extension_list);
         return true;
     }
     if (name == "library_list" && type == TypeString) {
         if (library_list.empty())
-            pvt::catalog_all_plugins(plugin_searchpath.string());
+            OIIO::pvt::catalog_all_plugins(plugin_searchpath.string());
         *(ustring*)val = ustring(library_list);
         return true;
     }
@@ -684,8 +718,11 @@ getattribute(string_view name, TypeDesc type, void* val)
     return false;
 }
 
+OIIO_NAMESPACE_3_1_END
 
 
+OIIO_NAMESPACE_BEGIN
+
 namespace {
 
 /// Type-independent template for turning potentially
@@ -736,11 +773,37 @@ _contiguize(const T* src, int nchannels, stride_t xstride, stride_t ystride,
 
 }  // namespace
 
+
+
+span<const std::byte>
+pvt::contiguize(const image_span<const std::byte>& src, span<std::byte> dst)
+{
+    // Contiguized result must fit in dst
+    OIIO_DASSERT(src.size_bytes() <= dst.size_bytes());
+
+    if (src.is_contiguous()) {
+        // Already contiguous -- don't copy, just return a span representation
+        // of the place it already lives in src.
+        return make_cspan(src.data(), src.size_bytes());
+    } else {
+        // Non-contiguous -- rely on copy_image() to do the heavy lifting.
+        image_span dstspan(dst.data(), src.nchannels(), src.width(),
+                           src.height(), src.depth(), AutoStride, AutoStride,
+                           AutoStride, AutoStride, src.chansize());
+        OIIO_DASSERT(dstspan.size_bytes() == src.size_bytes());
+        copy_image(dstspan, src);
+        return make_cspan(dst.data(), src.size_bytes());
+    }
+}
+
+
+
 const void*
 pvt::contiguize(const void* src, int nchannels, stride_t xstride,
                 stride_t ystride, stride_t zstride, void* dst, int width,
                 int height, int depth, TypeDesc format)
 {
+    OIIO_DASSERT(nchannels >= 0 && width >= 0 && height >= 0 && depth >= 0);
     switch (format.basetype) {
     case TypeDesc::FLOAT:
         return _contiguize((const float*)src, nchannels, xstride, ystride,
@@ -851,6 +914,10 @@ pvt::parallel_convert_from_float(const float* src, void* dst, size_t nvals,
     return dst;
 }
 
+OIIO_NAMESPACE_END
+
+
+OIIO_NAMESPACE_3_1_BEGIN
 
 
 bool
@@ -865,7 +932,7 @@ convert_pixel_values(TypeDesc src_type, const void* src, TypeDesc dst_type,
 
     if (dst_type == TypeFloat) {
         // Special case -- converting non-float to float
-        pvt::convert_to_float(src, (float*)dst, n, src_type);
+        OIIO::pvt::convert_to_float(src, (float*)dst, n, src_type);
         return true;
     }
 
@@ -881,7 +948,7 @@ convert_pixel_values(TypeDesc src_type, const void* src, TypeDesc dst_type,
             tmp.reset(new float[n]);  // Freed when tmp exists its scope
             buf = tmp.get();
         }
-        pvt::convert_to_float(src, buf, n, src_type);
+        OIIO::pvt::convert_to_float(src, buf, n, src_type);
     }
 
     // Convert float to 'dst_type'
@@ -1030,6 +1097,119 @@ copy_image(int nchannels, int width, int height, int depth, const void* src,
 
 
 
+template<typename T>
+void
+aligned_copy_image(const image_span<std::byte>& dst,
+                   const image_span<const std::byte>& src)
+{
+    size_t systride  = src.ystride();
+    size_t dystride  = dst.ystride();
+    size_t chunksize = src.chansize() * src.nchannels();
+    size_t nT        = chunksize / sizeof(T);
+    size_t sstrideT  = src.xstride() / sizeof(T);
+    size_t dstrideT  = dst.xstride() / sizeof(T);
+    for (uint32_t z = 0; z < src.depth(); ++z) {
+        std::byte* dscanline       = dst.getptr(0, 0, 0, z);
+        const std::byte* sscanline = src.getptr(0, 0, 0, z);
+        for (uint32_t y = 0; y < src.height(); ++y) {
+            auto dscanlineT = reinterpret_cast<T*>(dscanline);
+            auto sscanlineT = reinterpret_cast<const T*>(sscanline);
+            for (uint32_t x = 0; x < src.width(); ++x) {
+                for (size_t c = 0; c < nT; ++c) {
+                    dscanlineT[x * dstrideT + c] = sscanlineT[x * sstrideT + c];
+                }
+            }
+            sscanline += systride;
+            dscanline += dystride;
+        }
+    }
+}
+
+
+
+bool
+copy_image(const image_span<std::byte>& dst,
+           const image_span<const std::byte>& src)
+{
+    OIIO_DASSERT(src.width() == dst.width() && src.height() == dst.height()
+                 && src.depth() == dst.depth()
+                 && src.nchannels() == dst.nchannels()
+                 && src.chansize() == dst.chansize());
+    if (src.is_contiguous() && dst.is_contiguous()) {
+        // Whole image is contiguous -- just do it in one memcpy
+        memcpy(dst.data(), src.data(), src.size_bytes());
+        return true;
+    }
+    if (src.is_contiguous_scanline() && dst.is_contiguous_scanline()) {
+        // Scanlines are contiguous -- do one memcpy per scanline
+        size_t chunksize = src.chansize() * src.nchannels()
+                           * size_t(src.width());
+        for (uint32_t z = 0; z < src.depth(); ++z) {
+            for (uint32_t y = 0; y < src.height(); ++y) {
+                std::byte* d       = dst.getptr(0, 0, y, z);
+                const std::byte* s = src.getptr(0, 0, y, z);
+                memcpy(d, s, chunksize);
+            }
+        }
+        return true;
+    }
+    if (dst.is_contiguous_pixel()) {
+        size_t systride = src.ystride();
+        size_t dystride = dst.ystride();
+        // Pixels are contiguous -- do one memcpy per pixel
+        size_t chunksize = src.chansize() * src.nchannels();
+#if 1
+        // Speedup trick: if we 'or' all the pointers and strides and the
+        // result is a multiple of 4, we can copy 4 bytes at a time.
+        size_t mashed = (chunksize | uintptr_t(src.data())
+                         | uintptr_t(dst.data()) | src.xstride() | src.ystride()
+                         | src.zstride());
+        if ((mashed & 3) == 0) {
+            aligned_copy_image<uint32_t>(dst, src);
+            return true;
+        }
+        if ((mashed & 1) == 0) {
+            aligned_copy_image<uint16_t>(dst, src);
+            return true;
+        }
+#endif
+        for (uint32_t z = 0; z < src.depth(); ++z) {
+            std::byte* dscanline       = dst.getptr(0, 0, 0, z);
+            const std::byte* sscanline = src.getptr(0, 0, 0, z);
+            for (uint32_t y = 0; y < src.height(); ++y) {
+                for (uint32_t x = 0; x < src.width(); ++x) {
+                    memcpy(dscanline + x * dst.xstride(),
+                           sscanline + x * src.xstride(), chunksize);
+                }
+                sscanline += systride;
+                dscanline += dystride;
+            }
+        }
+        return true;
+    }
+    // General case -- have to do item by item copy.
+    // FIXME: is there advantage to doing copies individually rather
+    // then with memcpy?
+    size_t chunksize = src.chansize();
+    for (uint32_t z = 0; z < src.depth(); ++z) {
+        for (uint32_t y = 0; y < src.height(); ++y) {
+            std::byte* dscanline       = dst.getptr(0, 0, y, z);
+            const std::byte* sscanline = src.getptr(0, 0, y, z);
+            for (uint32_t x = 0; x < src.width(); ++x) {
+                std::byte* dpel       = dscanline + x * dst.xstride();
+                const std::byte* spel = sscanline + x * src.xstride();
+                for (uint32_t c = 0; x < src.nchannels(); ++c) {
+                    memcpy(dpel + c * dst.chanstride(),
+                           spel + c * src.chanstride(), chunksize);
+                }
+            }
+        }
+    }
+    return true;
+}
+
+
+
 void
 add_bluenoise(int nchannels, int width, int height, int depth, float* data,
               stride_t xstride, stride_t ystride, stride_t zstride,
@@ -1050,10 +1230,9 @@ add_bluenoise(int nchannels, int width, int height, int depth, float* data,
                     int channel = c + chorigin;
                     if (channel == alpha_channel || channel == z_channel)
                         continue;
-                    float dither
-                        = pvt::bluenoise_4chan_ptr(x + xorigin, y + yorigin,
-                                                   z + zorigin, channel & (~3),
-                                                   ditherseed)[channel & 3];
+                    float dither = OIIO::pvt::bluenoise_4chan_ptr(
+                        x + xorigin, y + yorigin, z + zorigin, channel & (~3),
+                        ditherseed)[channel & 3];
                     *val += ditheramplitude * (dither - 0.5f);
                 }
             }
@@ -1222,4 +1401,45 @@ wrap_mirror(int& coord, int origin, int width)
 }
 
 
-OIIO_NAMESPACE_END
+
+/// Verify that the image_span has all its contents lying within the
+/// contiguous span.
+bool
+image_span_within_span(const image_span<const std::byte>& ispan,
+                       span<const std::byte> contiguous) noexcept
+{
+    // Start with first,last being the first byte of pixel 0, channel 0
+    const std::byte* first = ispan.data();  // first byte of ispan
+    const std::byte* last  = ispan.data();  // last byte of ispan
+    // Extend them to the start of the first pixel of the first and last image
+    // plane.
+    if (ispan.zstride() > 0)
+        last += ispan.zstride() * (ispan.depth() - 1);
+    else
+        first += ispan.zstride() * (ispan.depth() - 1);  // neg stride
+    // Extend them to the start of the first pixel of the first and last
+    // scanline of the first and last image plane.
+    if (ispan.ystride() > 0)
+        last += ispan.ystride() * (ispan.height() - 1);
+    else
+        first += ispan.ystride() * (ispan.height() - 1);  // neg stride
+    // Extend them to the start of the first pixel of the first and last
+    // column of the first and last scanline of the first and last image
+    // plane.
+    if (ispan.xstride() > 0)
+        last += ispan.xstride() * (ispan.width() - 1);
+    else
+        first += ispan.xstride() * (ispan.width() - 1);  // neg stride
+    // Make sure they cover the whole of those extreme pixels
+    if (ispan.chanstride() > 0)
+        last += ispan.chanstride() * (ispan.nchannels() - 1);
+    else
+        first += ispan.chanstride() * (ispan.nchannels() - 1);  // neg stride
+    // Make sure last covers the whole data type
+    last += ispan.chansize() - 1;
+    return (first >= contiguous.data()
+            && last < contiguous.data() + contiguous.size());
+}
+
+
+OIIO_NAMESPACE_3_1_END
