@@ -36,6 +36,25 @@ command += info_command ("allhalf.exr", safematch=1)
 command += oiiotool ("src/rgbaz.exr -d half -d Z=float -o rgbahalf-zfloat.exr")
 command += info_command ("rgbahalf-zfloat.exr", safematch=1)
 
+# test -d SUBIMAGE.NAME=fmt to change data format of one subimage
+command += oiiotool ("--create 64x64 3 --attrib oiio:subimagename subimageA " +
+                     "--create 64x64 3 --attrib oiio:subimagename subimageB " +
+                     "--create 64x64 4 --attrib oiio:subimagename subimageC " +
+                     "--siappendall -d half -d subimageB.G=float -o sub-formats.exr")
+command += info_command ("sub-formats.exr", extraargs="--metamatch subimages")
+# test -d SUBIMAGE.*=fmt to change data format of one channel of one subimage
+command += oiiotool ("--create 64x64 3 --attrib oiio:subimagename subimageA " +
+                     "--create 64x64 3 --attrib oiio:subimagename subimageB " +
+                     "--create 64x64 4 --attrib oiio:subimagename subimageC " +
+                     "--siappendall -d half -d subimageB.*=float -o sub-formats2.exr")
+command += info_command ("sub-formats2.exr", extraargs="--metamatch subimages")
+# test -d SUBIMAGE.*=fmt to change data format of one channel of one subimage,
+# when no subimages are named (they wil be upon output) but it will still
+# recognize "subimage{:02}".
+command += oiiotool ("--create 64x64 3 --dup --dup " +
+                     "--siappendall -d half -d subimage01.*=float -o sub-formats3.exr")
+command += info_command ("sub-formats3.exr", extraargs="--metamatch subimages")
+
 
 # Some tests to verify that we are transferring data formats properly.
 #
